# Copyright 2023 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.


from .base import fire_opal_workflow


@fire_opal_workflow("mitigation_workflow")
def create_mitigation_data(
    ibm_device_name: str,
    ibmq_token: str,
    hub: str,
    group: str,
    project: str,
):
    """Creates mitigation data for Fire Opal."""

    return {
        "ibm_device_name": ibm_device_name,
        "ibmq_token": ibmq_token,
        "hub": hub,
        "group": group,
        "project": project,
    }



# Copyright 2023 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.

from typing import (
    Any,
    Dict,
    List,
)

from qctrlclient.core import print_warnings

from .base import fire_opal_workflow


@fire_opal_workflow("compile_and_run_workflow", formatter=print_warnings)
def execute(
    circuits: List[str],
    shot_count: int,
    credentials: Dict[str, Any],
    backend_name: str,
):
    """
    Execute a batch of `circuits` where `shot_count` measurements are taken per circuit.

    Parameters
    ----------
    circuits : List[str]
        A list of quantum circuits in the form of a QASM strings. You may use Qiskit to
        generate these strings.
    shot_count : int
        Number of bitstrings that are sampled from the final quantum state.
    credentials : Dict[str, Any]
        The credentials for running circuits on an IBM backend. This dictionary should
        contain key-value entries with keys `token`, `project`, `hub`, and `group`.
    backend_name : str
        The backend device name that should be used to run circuits.
    """
    return {
        "circuits": circuits,
        "shot_count": shot_count,
        "credentials": credentials,
        "backend_name": backend_name,
    }



# Copyright 2023 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.

from typing import (
    Dict,
    Optional,
    Union,
)

import networkx as nx
from qctrlclient.core import print_warnings
from sympy import Expr

from .base import fire_opal_workflow


@fire_opal_workflow("solve_qaoa_workflow", formatter=print_warnings)
def solve_qaoa(
    problem: Union[Expr, nx.Graph],
    problem_type: str,
    credentials: Dict[str, str],
    backend_name: Optional[str] = None,
):
    """
    Solve a QAOA problem.

    Parameters
    ----------
    problem : Union[Expr, nx.Graph]
        The QAOA problem definition, represented as either an
        `nx.Graph` or `sympy.Expr`.
    problem_type : str
        The class of QAOA problem to solve.
    credentials : Dict[str, str]
        The credentials for running circuits on an IBM backend. This dictionary should
        contain key-value entries with keys `token`, `project`, `hub`, and `group`.
    backend_name : str, optional
        The backend device that should be used to run circuits. Defaults to None.
    """
    print(
        "This function performs multiple circuit executions. "
        "The total execution time is expected to take a few minutes but will take "
        "much longer if waiting in the public queue. It is highly recommended to visit "
        "https://quantum-computing.ibm.com/reservations and make a dedicated reservation.\n"
    )
    print("Visit https://quantum-computing.ibm.com/jobs to check your queue position.")
    return {
        "problem": problem,
        "problem_type": problem_type,
        "credentials": credentials,
        "backend_name": backend_name,
    }



# Copyright 2023 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.


from typing import List

from .base import fire_opal_workflow


@fire_opal_workflow("calibration_workflow")
def create_calibration_data(
    ibm_device_name: str,
    gate_type: str,
    echo: bool,
    ibmq_token: str,
    hub: str,
    group: str,
    project: str,
    gates_to_calibrate: List[str],
    optimizer_iterations: int = 10,
    candidate_count: int = 4,
    cost_temperature: float = 0.5,
    temperature_scaling: float = 0.5,
    use_latest_gates: bool = False,
    elite_fraction: float = 0.15,
    optimizer_choice: str = "simulated_annealing",
):
    """Creates calibration data for Fire Opal."""

    return {
        "ibm_device_name": ibm_device_name,
        "gate_type": gate_type,
        "echo": echo,
        "ibmq_token": ibmq_token,
        "hub": hub,
        "group": group,
        "project": project,
        "gates_to_calibrate": gates_to_calibrate,
        "optimizer_iterations": optimizer_iterations,
        "candidate_count": candidate_count,
        "cost_temperature": cost_temperature,
        "temperature_scaling": temperature_scaling,
        "use_latest_gates": use_latest_gates,
        "elite_fraction": elite_fraction,
        "optimizer_choice": optimizer_choice,
    }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fireopal', 'fireopal.functions']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.0,<9.0.0',
 'qctrl-client>=7.0.0,<8.0.0',
 'qctrl-commons>=18.0.0,<19.0.0']

setup_kwargs = {
    'name': 'fire-opal',
    'version': '5.2.0',
    'description': 'Fire Opal Client',
    'long_description': "# Fire Opal Client\n\nThe Fire Opal Client package is a Python client for Q-CTRL's Fire Opal product.\nFire Opal is a simple and powerful package for algorithm developers and quantum computer end users.\nBy applying a complete suite of error suppression techniques, Fire Opal automatically reduces error and vastly improves the quality of quantum algorithm results. This often transforms quantum computer outputs from random to useful.\n\nPlease see how you can [get started](https://docs.q-ctrl.com/fire-opal/get-started) today.\n",
    'author': 'Q-CTRL',
    'author_email': 'support@q-ctrl.com',
    'maintainer': 'Q-CTRL',
    'maintainer_email': 'support@q-ctrl.com',
    'url': 'https://q-ctrl.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.12',
}


setup(**setup_kwargs)



# Copyright 2023 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.

from qctrlclient import (
    CliAuth,
    GraphQLClient,
    get_default_api_url,
    get_default_cli_auth,
)
from qctrlclient.core import (
    ApiRouter,
    CoreClientSettings,
    LocalRouter,
    Product,
)
from qctrlclient.core.utils import show_error_message
from qctrlclient.exceptions import GraphQLClientError
from qctrlclient.globals import global_value

from .constants import INVALID_SUBSCRIPTION_ERROR


def _get_graphql_client(api_url: str, auth: CliAuth) -> GraphQLClient:
    """Returns a `GraphQLClient` and checks the user
    has the required access for CLI usage.
    """
    client = GraphQLClient(api_url, auth=auth)

    try:
        client.check_user_role("fire-opal-cli-access")
    except GraphQLClientError:
        show_error_message(INVALID_SUBSCRIPTION_ERROR)

    return client


def get_default_router() -> ApiRouter:
    """Returns the default router that the Fire Opal
    client uses.
    """
    client = _get_graphql_client(get_default_api_url(), get_default_cli_auth())
    settings = get_config()
    return ApiRouter(client, settings)


@global_value("FIRE_OPAL_CONFIG")
def get_config() -> CoreClientSettings:
    """Returns the global Fire Opal settings."""
    return CoreClientSettings(router=get_default_router, product=Product.FIRE_OPAL)


def configure(**kwargs):
    """Updates the global Fire Opal settings. See `CoreClientSettings`
    for details on which fields can be updated.
    """
    config = get_config()
    config.update(**kwargs)


def configure_api(api_url: str, oidc_url: str):
    """Convenience function to configure Fire Opal for API
    routing.

    Parameters
    ----------
    api_url : str
        URL of the GraphQL schema
    oidc_url : str
        Base URL of the OIDC provider e.g. Keycloak
    """
    client = _get_graphql_client(api_url, CliAuth(oidc_url))
    settings = get_config()

    configure(router=ApiRouter(client, settings))


def configure_local(resolver: "BaseResolver"):
    """Convenience function to configure Fire Opal for local
    routing.

    Parameters
    ----------
    resolver : BaseResolver
        A local implementation of a workflow resolver which uses
        a registry that implements all of the available Fire Opal
        workflows
    """
    configure(router=LocalRouter(resolver))


def configure_organization(organization_slug: str):
    """Convenience function to configure the organization.

    Parameters
    ----------
    organization_slug : str
        Unique slug for the organization.
    """
    configure(organization=organization_slug)



# Copyright 2023 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.

from .benchmark import benchmark
from .create_calibration_data import create_calibration_data
from .create_mitigation_data import create_mitigation_data
from .execute import execute
from .read_data import read_data
from .show_supported_devices import show_supported_devices
from .solve_qaoa import solve_qaoa
from .validate import validate
